-module(advanced_ai_player).
-behaviour(gen_server).

-export([start_link/2, init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).
-export([play_turn/1, get_stats/1]).

-record(state, {
    name,
    personality,      % aggressive | conservative | balanced | adaptive
    learning_model,   % 机器学习模型引用
    game_history = [],
    play_stats = #{},
    current_game = undefined,
    adaptation_level = 0.0
}).

%% 性格特征定义
-define(PERSONALITY_TRAITS, #{
    aggressive => #{
        risk_tolerance => 0.8,
        combo_preference => 0.7,
        control_value => 0.6
    },
    conservative => #{
        risk_tolerance => 0.3,
        combo_preference => 0.4,
        control_value => 0.8
    },
    balanced => #{
        risk_tolerance => 0.5,
        combo_preference => 0.5,
        control_value => 0.5
    },
    adaptive => #{
        risk_tolerance => 0.5,
        combo_preference => 0.5,
        control_value => 0.5
    }
}).

%% API
start_link(Name, Personality) ->
    gen_server:start_link(?MODULE, [Name, Personality], []).

play_turn(Pid) ->
    gen_server:cast(Pid, play_turn).

get_stats(Pid) ->
    gen_server:call(Pid, get_stats).

%% Callbacks
init([Name, Personality]) ->
    {ok, LearningModel} = ml_engine:start_link(),
    {ok, #state{
        name = Name,
        personality = Personality,
        learning_model = LearningModel
    }}.

handle_cast(play_turn, State) ->
    {Play, NewState} = calculate_best_move(State),
    execute_play(Play, NewState),
    {noreply, NewState}.

handle_call(get_stats, _From, State) ->
    Stats = compile_statistics(State),
    {reply, {ok, Stats}, State}.

%% 高级AI策略实现

calculate_best_move(State) ->
    GameState = analyze_game_state(State),
    Personality = get_personality_traits(State),
    
    % 获取各种可能的动作
    PossiblePlays = generate_possible_plays(State),
    
    % 使用机器学习模型评估每个动作
    RatedPlays = evaluate_plays(PossiblePlays, GameState, State),
    
    % 根据性格特征调整评分
    AdjustedPlays = adjust_by_personality(RatedPlays, Personality, State),
    
    % 选择最佳动作
    BestPlay = select_best_play(AdjustedPlays, State),
    
    % 更新状态和学习模型
    NewState = update_state_and_learn(State, BestPlay, GameState),
    
    {BestPlay, NewState}.

analyze_game_state(State) ->
    % 分析当前游戏状态
    #{
        cards_in_hand => get_cards_in_hand(State),
        cards_played => get_cards_played(State),
        opponent_info => analyze_opponents(State),
        game_stage => determine_game_stage(State),
        control_status => analyze_control(State)
    }.

generate_possible_plays(State) ->
    Cards = get_cards_in_hand(State),
    LastPlay = get_last_play(State),
    
    % 生成所有可能的牌组合
    AllCombinations = card_rules:generate_combinations(Cards),
    
    % 过滤出合法的出牌选择
    ValidPlays = filter_valid_plays(AllCombinations, LastPlay),
    
    % 添加"不出"选项
    [pass | ValidPlays].

evaluate_plays(Plays, GameState, State) ->
    lists:map(
        fun(Play) ->
            Score = evaluate_single_play(Play, GameState, State),
            {Play, Score}
        end,
        Plays
    ).

evaluate_single_play(Play, GameState, State) ->
    % 使用机器学习模型评估
    Features = extract_features(Play, GameState),
    {ok, BaseScore} = ml_engine:predict(State#state.learning_model, Features),
    
    % 考虑多个因素
    ControlScore = evaluate_control_value(Play, GameState),
    TempoScore = evaluate_tempo_value(Play, GameState),
    RiskScore = evaluate_risk_value(Play, GameState),
    
    % 综合评分
    BaseScore * 0.4 + ControlScore * 0.3 + TempoScore * 0.2 + RiskScore * 0.1.

adjust_by_personality(RatedPlays, Personality, State) ->
    RiskTolerance = maps:get(risk_tolerance, Personality),
    ComboPreference = maps:get(combo_preference, Personality),
    ControlValue = maps:get(control_value, Personality),
    
    lists:map(
        fun({Play, Score}) ->
            AdjustedScore = adjust_score_by_traits(Score, Play, RiskTolerance, 
                                                 ComboPreference, ControlValue, State),
            {Play, AdjustedScore}
        end,
        RatedPlays
    ).

select_best_play(AdjustedPlays, State) ->
    % 根据评分选择最佳动作，但加入一些随机性以避免过于可预测
    case State#state.personality of
        adaptive ->
            select_adaptive_play(AdjustedPlays, State);
        _ ->
            select_personality_based_play(AdjustedPlays, State)
    end.

update_state_and_learn(State, Play, GameState) ->
    % 记录动作
    NewHistory = [Play | State#state.game_history],
    
    % 更新统计信息
    NewStats = update_play_stats(State#state.play_stats, Play),
    
    % 如果是自适应性格，更新适应级别
    NewAdaptationLevel = case State#state.personality of
        adaptive ->
            update_adaptation_level(State#state.adaptation_level, Play, GameState);
        _ ->
            State#state.adaptation_level
    end,
    
    % 更新机器学习模型
    Features = extract_features(Play, GameState),
    Reward = calculate_play_reward(Play, GameState),
    ml_engine:update_model(State#state.learning_model, Features, Reward),
    
    State#state{
        game_history = NewHistory,
        play_stats = NewStats,
        adaptation_level = NewAdaptationLevel
    }.