-module(ai_strategy).
-export([initialize_strategy/0, update_strategy/2, make_decision/2, analyze_game_state/1, evaluate_play/3,
         generate_possible_plays/1, generate_singles/1, generate_pairs/1, generate_triples/1, generate_straights/1, generate_bombs/1]).

-record(game_state, {
    hand_cards,          % 手牌
    played_cards = [],   % 已出牌
    player_position,     % 玩家位置（地主/农民）
    remaining_cards,     % 剩余牌数
    stage,              % 游戏阶段
    last_play,          % 上家出牌
    control_status      % 是否控制局面
}).

%% 策略权重配置
-define(STRATEGY_WEIGHTS, #{
    early_game => #{
        control_weight => 0.7,
        combo_weight => 0.5,
        defensive_weight => 0.3,
        risk_weight => 0.4
    },
    mid_game => #{
        control_weight => 0.5,
        combo_weight => 0.6,
        defensive_weight => 0.5,
        risk_weight => 0.5
    },
    late_game => #{
        control_weight => 0.3,
        combo_weight => 0.8,
        defensive_weight => 0.7,
        risk_weight => 0.6
    }
}).

%% 初始化策略系统
initialize_strategy() ->
    #{
        weights => ?STRATEGY_WEIGHTS,
        learning_rate => 0.01,
        adaptation_rate => 0.1,
        experience => #{},
        history => []
    }.

%% 更新策略
update_strategy(Strategy, GameState) ->
    Stage = determine_game_stage(GameState),
    UpdatedWeights = adjust_weights(Strategy, Stage, GameState),
    NewExperience = update_experience(Strategy, GameState),
    Strategy#{
        weights => UpdatedWeights,
        experience => NewExperience,
        history => [GameState | maps:get(history, Strategy)]
    }.

%% 做出决策
make_decision(Strategy, GameState) ->
    PossiblePlays = generate_possible_plays(GameState#game_state.hand_cards),
    EvaluatedPlays = evaluate_all_plays(PossiblePlays, Strategy, GameState),
    select_best_play(EvaluatedPlays, Strategy, GameState).

%% 评估所有可能的出牌组合生成函数的具体实现
generate_singles(Cards) ->
    lists:map(fun(C) -> [C] end, Cards).

generate_pairs(Cards) ->
    UniqueCards = lists:usort(Cards),
    [lists:duplicate(2, C) || C <- UniqueCards, length([C || C1 <- Cards, C1 == C]) >= 2].

generate_triples(Cards) ->
    UniqueCards = lists:usort(Cards),
    [lists:duplicate(3, C) || C <- UniqueCards, length([C || C1 <- Cards, C1 == C]) >= 3].

generate_straights(Cards) ->
    Sorted = lists:sort(Cards),
    find_straights(Sorted, 5).

find_straights(_, MinLen) when MinLen < 5 -> [];
find_straights([H|T], Len) ->
    case lists:seq(H, H+Len-1) of
        Seq when length(Seq) == Len -> [Seq];
        _ -> find_straights(T, Len)
    end.

generate_bombs(Cards) ->
    [lists:duplicate(4, C) || C <- lists:usort(Cards), length([C || C1 <- Cards, C1 == C]) >= 4].

%% 生成所有可能的出牌
generate_possible_plays(Cards) ->
    Singles = generate_singles(Cards),
    Pairs = generate_pairs(Cards),
    Triples = generate_triples(Cards),
    Straights = generate_straights(Cards),
    Bombs = generate_bombs(Cards),
    Singles ++ Pairs ++ Triples ++ Straights ++ Bombs.

%% 游戏阶段判断函数
determine_game_stage(GameState) ->
    % 简单根据剩余牌数判断阶段
    case GameState#game_state.remaining_cards of
        N when N > 20 -> early_game;
        N when N > 10 -> mid_game;
        _ -> late_game
    end.

%% 调整权重
adjust_weights(Strategy, Stage, GameState) ->
    CurrentWeights = maps:get(weights, Strategy),
    StageWeights = maps:get(Stage, CurrentWeights),
    % 根据游戏状态微调权重
    maps:map(
        fun(Key, Value) ->
            AdjustmentFactor = calculate_adjustment_factor(Key, GameState),
            Value * AdjustmentFactor
        end,
        StageWeights
    ).

%% 计算调整因子
calculate_adjustment_factor(WeightKey, GameState) ->
    % 简单实现，根据不同权重类型返回调整因子
    case WeightKey of
        control_weight when GameState#game_state.control_status -> 1.2;
        control_weight -> 0.8;
        _ -> 1.0
    end.

%% 更新经验
update_experience(Strategy, GameState) ->
    Experience = maps:get(experience, Strategy),
    % 简单实现，记录游戏状态特征
    Experience#{
        GameState#game_state.stage => maps:get(GameState#game_state.stage, Experience, 0) + 1
    }.

%% 评估所有出牌
evaluate_all_plays(Plays, Strategy, GameState) ->
    [{Play, evaluate_play(Play, Strategy, GameState)} || Play <- Plays].

%% 评估单个出牌
evaluate_play(Play, Strategy, GameState) ->
    % 简单评估逻辑
    ControlScore = evaluate_control_impact(Play, GameState),
    ComboScore = evaluate_combo_potential(Play, GameState),
    DefensiveScore = evaluate_defensive_value(Play, GameState),
    
    % 获取当前阶段权重
    Stage = GameState#game_state.stage,
    Weights = maps:get(Stage, maps:get(weights, Strategy)),
    
    % 计算加权得分
    ControlWeight = maps:get(control_weight, Weights),
    ComboWeight = maps:get(combo_weight, Weights),
    DefensiveWeight = maps:get(defensive_weight, Weights),
    
    (ControlScore * ControlWeight) + (ComboScore * ComboWeight) + (DefensiveScore * DefensiveWeight).

%% 评估控制影响
evaluate_control_impact(Play, GameState) ->
    % 简单实现
    case length(Play) of
        1 -> 0.3; % 单牌控制性较低
        2 -> 0.5; % 对子中等
        3 -> 0.7; % 三张较高
        4 when Play == [Play|_] -> 1.0; % 炸弹最高
        _ -> 0.6 % 其他牌型
    end.

%% 评估连招潜力
evaluate_combo_potential(Play, GameState) ->
    % 简单实现
    0.5. % 默认中等潜力

%% 评估防守价值
evaluate_defensive_value(Play, GameState) ->
    % 简单实现
    0.5. % 默认中等防守价值

%% 选择最佳出牌
select_best_play(EvaluatedPlays, Strategy, GameState) ->
    case EvaluatedPlays of
        [] -> pass;
        _ ->
            {BestPlay, _Score} = lists:max(fun({_, ScoreA}, {_, ScoreB}) -> ScoreA >= ScoreB end, EvaluatedPlays),
            BestPlay
    end.

%% 分析游戏状态
analyze_game_state(GameState) ->
    #{
        hand_strength => calculate_hand_strength(GameState#game_state.hand_cards),
        control_level => calculate_control_level(GameState),
        stage => GameState#game_state.stage,
        position_advantage => calculate_position_advantage(GameState)
    }.

%% 计算手牌强度
calculate_hand_strength(Cards) ->
    % 简单实现，根据牌型数量和质量评估
    Bombs = length(generate_bombs(Cards)),
    Triples = length(generate_triples(Cards)),
    Pairs = length(generate_pairs(Cards)),
    
    (Bombs * 0.5) + (Triples * 0.3) + (Pairs * 0.2).

%% 计算控制水平
calculate_control_level(GameState) ->
    % 简单实现
    case GameState#game_state.control_status of
        true -> 0.8;
        false -> 0.2
    end.

%% 计算位置优势
calculate_position_advantage(GameState) ->
    % 简单实现，地主位置优势较大
    case GameState#game_state.player_position of
        landlord -> 0.7;
        farmer -> 0.3
    end.