-module(doudizhu_ai).
-behaviour(gen_server).

-export([
    start_link/1,
    make_decision/2,
    analyze_cards/1,
    update_game_state/2
]).

-export([
    init/1,
    handle_call/3,
    handle_cast/2,
    handle_info/2,
    terminate/2,
    code_change/3
]).

-include("card_types.hrl").

-record(state, {
    player_id,           % AI 玩家ID
    role,               % dizhu | nongmin (地主或农民)
    known_cards = [],   % 已知的牌
    hand_cards = [],    % 手牌
    played_cards = [],  % 已打出的牌
    other_players = [], % 其他玩家信息
    game_history = [],  % 游戏历史
    strategy_cache = #{} % 策略缓存
}).

%% API 函数
start_link(PlayerId) ->
    gen_server:start_link({local, ?MODULE}, ?MODULE, [PlayerId], []).

make_decision(GameState, Options) ->
    gen_server:call(?MODULE, {make_decision, GameState, Options}).

analyze_cards(Cards) ->
    gen_server:call(?MODULE, {analyze_cards, Cards}).

update_game_state(Event, Data) ->
    gen_server:cast(?MODULE, {update_game_state, Event, Data}).

%% Callback 函数
init([PlayerId]) ->
    {ok, #state{player_id = PlayerId}}.

handle_call({make_decision, GameState, Options}, _From, State) ->
    {Decision, NewState} = calculate_best_move(GameState, Options, State),
    {reply, Decision, NewState};

handle_call({analyze_cards, Cards}, _From, State) ->
    Analysis = perform_cards_analysis(Cards, State),
    {reply, Analysis, State};

handle_call(_Request, _From, State) ->
    {reply, ok, State}.

handle_cast({update_game_state, Event, Data}, State) ->
    NewState = update_state(Event, Data, State),
    {noreply, NewState};

handle_cast(_Msg, State) ->
    {noreply, State}.

handle_info(_Info, State) ->
    {noreply, State}.

terminate(_Reason, _State) ->
    ok.

code_change(_OldVsn, State, _Extra) ->
    {ok, State}.

%% 内部函数

calculate_best_move(GameState, Options, State) ->
    % 1. 分析当前局势
    Situation = analyze_situation(GameState, State),
    % 2. 生成可能的动作
    PossibleMoves = generate_possible_moves(GameState, Options, State),
    % 3. 评估每个动作
    ScoredMoves = evaluate_moves(PossibleMoves, Situation, State),
    % 4. 选择最佳动作
    {BestMove, Score} = select_best_move(ScoredMoves),
    % 5. 更新状态
    NewState = update_strategy_state(BestMove, Score, State),
    {BestMove, NewState}.

analyze_situation(GameState, State) ->
    #situation{
        game_stage = determine_game_stage(GameState),
        hand_strength = evaluate_hand_strength(State#state.hand_cards),
        control_level = calculate_control_level(GameState, State),
        winning_probability = estimate_winning_probability(GameState, State)
    }.

%% 确定游戏阶段
determine_game_stage(GameState) ->
    CardsLeft = count_remaining_cards(GameState),
    cond do
        CardsLeft > 15 -> early_game;
        CardsLeft > 8 -> mid_game;
        true -> end_game
    end.

%% 评估手牌强度
evaluate_hand_strength(Cards) ->
    % 分析手牌组成
    Components = analyze_card_components(Cards),
    % 计算基础分数
    BaseScore = calculate_base_score(Components),
    % 计算组合价值
    ComboScore = calculate_combo_value(Components),
    % 计算控制牌价值
    ControlScore = calculate_control_value(Components),
    % 返回综合评分
    #hand_strength{
        base_score = BaseScore,
        combo_score = ComboScore,
        control_score = ControlScore,
        total_score = BaseScore + ComboScore + ControlScore
    }.

%% 分析牌型组件
analyze_card_components(Cards) ->
    % 对牌进行分组
    Groups = group_cards_by_value(Cards),
    % 识别各种牌型
    Singles = find_singles(Groups),
    Pairs = find_pairs(Groups),
    Triples = find_triples(Groups),
    Bombs = find_bombs(Groups),
    Sequences = find_sequences(Groups),
    % 返回组件分析结果
    #components{
        singles = Singles,
        pairs = Pairs,
        triples = Triples,
        bombs = Bombs,
        sequences = Sequences
    }.

%% 生成可能的出牌选择
generate_possible_moves(GameState, Options, State) ->
    LastPlay = get_last_play(GameState),
    HandCards = State#state.hand_cards,
    case LastPlay of
        none ->
            generate_leading_moves(HandCards);
        {Type, Value, _Cards} ->
            generate_following_moves(HandCards, Type, Value)
    end.

%% 评估每个可能的动作
evaluate_moves(Moves, Situation, State) ->
    lists:map(fun(Move) ->
        Score = calculate_move_score(Move, Situation, State),
        {Move, Score}
    end, Moves).

%% 计算单个动作的得分
calculate_move_score(Move, Situation, State) ->
    BaseScore = calculate_base_move_score(Move),
    PositionScore = calculate_position_score(Move, Situation),
    StrategyScore = calculate_strategy_score(Move, Situation, State),
    RiskScore = calculate_risk_score(Move, Situation, State),
    
    BaseScore * 0.4 + 
    PositionScore * 0.2 + 
    StrategyScore * 0.3 + 
    RiskScore * 0.1.

%% 选择最佳动作
select_best_move(ScoredMoves) ->
    lists:foldl(fun
        ({Move, Score}, {BestMove, BestScore}) when Score > BestScore ->
            {Move, Score};
        (_, Current) ->
            Current
    end, {pass, 0}, ScoredMoves).

%% 更新策略状态
update_strategy_state(Move, Score, State) ->
    NewCache = update_strategy_cache(Move, Score, State#state.strategy_cache),
    State#state{strategy_cache = NewCache}.

%% 生成开局动作
generate_leading_moves(Cards) ->
    % 基于手牌生成所有可能的出牌选择
    Components = analyze_card_components(Cards),
    % 生成不同类型的出牌
    Singles = generate_single_moves(Components),
    Pairs = generate_pair_moves(Components),
    Triples = generate_triple_moves(Components),
    Sequences = generate_sequence_moves(Components),
    Bombs = generate_bomb_moves(Components),
    % 合并所有可能的出牌
    Singles ++ Pairs ++ Triples ++ Sequences ++ Bombs.

%% 生成跟牌动作
generate_following_moves(Cards, Type, MinValue) ->
    % 根据上家出牌类型生成可能的跟牌
    ValidMoves = find_valid_moves(Cards, Type, MinValue),
    % 添加炸弹和火箭选项
    SpecialMoves = find_special_moves(Cards),
    ValidMoves ++ SpecialMoves.

%% 估算胜率
estimate_winning_probability(GameState, State) ->
    % 考虑多个因素计算胜率
    HandStrength = evaluate_hand_strength(State#state.hand_cards),
    Position = evaluate_position(GameState),
    RemainingCards = analyze_remaining_cards(GameState, State),
    ControlFactor = calculate_control_factor(GameState, State),
    
    BaseProb = calculate_base_probability(HandStrength, Position),
    AdjustedProb = adjust_probability(BaseProb, RemainingCards, ControlFactor),
    
    clamp(AdjustedProb, 0.0, 1.0).

%% 计算控制因子
calculate_control_factor(GameState, State) ->
    ControlCards = count_control_cards(State#state.hand_cards),
    TotalCards = count_total_cards(GameState),
    RemainingCards = count_remaining_cards(GameState),
    
    ControlRatio = ControlCards / max(1, RemainingCards),
    PositionBonus = calculate_position_bonus(GameState, State),
    
    ControlRatio * PositionBonus.

%% 分析剩余牌
analyze_remaining_cards(GameState, State) ->
    PlayedCards = get_played_cards(GameState),
    KnownCards = State#state.known_cards,
    AllCards = generate_full_deck(),
    
    RemainingCards = AllCards -- (PlayedCards ++ KnownCards),
    analyze_card_distribution(RemainingCards).

%% 更新游戏状态
update_state(Event, Data, State) ->
    case Event of
        play_cards ->
            update_after_play(Data, State);
        receive_cards ->
            update_after_receive(Data, State);
        game_over ->
            update_after_game_over(Data, State);
        _ ->
            State
    end.

%% 实用函数

clamp(Value, Min, Max) ->
    min(Max, max(Min, Value)).

%% 计算基础概率
calculate_base_probability(HandStrength, Position) ->
    BaseProb = HandStrength#hand_strength.total_score / 100,
    PositionMod = case Position of
        first -> 1.2;
        middle -> 1.0;
        last -> 0.8
    end,
    BaseProb * PositionMod.

%% 调整概率
adjust_probability(BaseProb, RemainingCards, ControlFactor) ->
    RemainingMod = calculate_remaining_modifier(RemainingCards),
    ControlMod = calculate_control_modifier(ControlFactor),
    
    BaseProb * RemainingMod * ControlMod.